@@ -31,11 +31,12 @@
 
 class QgsRectangle;
 class QgsCoordinateReferenceSystem;
+class QgsDataProviderTemporalCapabilities;
 
 
 /**
  * \ingroup core
- * Abstract base class for spatial data provider implementations.
+ * \brief Abstract base class for spatial data provider implementations.
  */
 class CORE_EXPORT QgsDataProvider : public QObject
 {
@@ -54,6 +55,10 @@ class CORE_EXPORT QgsDataProvider : public QObject
     {
       sipType = sipType_QgsMeshDataProvider;
     }
+    else if ( qobject_cast<QgsPointCloudDataProvider *>( sipCpp ) )
+    {
+      sipType = sipType_QgsPointCloudDataProvider;
+    }
     else
     {
       sipType = 0;
@@ -102,15 +107,33 @@ class CORE_EXPORT QgsDataProvider : public QObject
      */
     struct ProviderOptions
     {
+
+      /**
+       * Coordinate transform context
+       */
       QgsCoordinateTransformContext transformContext;
+
     };
 
+    /**
+     * Flags which control dataprovider construction.
+     * \since QGIS 3.16
+     */
+    enum ReadFlag
+    {
+      FlagTrustDataSource = 1 << 0, //!< Trust datasource config (primary key unicity, geometry type and srid, etc). Improves provider load time by skipping expensive checks like primary key unicity, geometry type and srid and by using estimated metadata on data load. Since QGIS 3.16
+      SkipFeatureCount = 1 << 1, //!< Make featureCount() return -1 to indicate unknown, and subLayers() to return a unknown feature count as well. Since QGIS 3.18. Only implemented by OGR provider at time of writing.
+    };
+    Q_DECLARE_FLAGS( ReadFlags, ReadFlag )
+
     /**
      * Create a new dataprovider with the specified in the \a uri.
      *
-     * Additional creation options are specified within the \a options value.
+     * Additional creation options are specified within the \a options value and since QGIS 3.16 creation flags are specified within the \a flags value.
      */
-    QgsDataProvider( const QString &uri = QString(), const QgsDataProvider::ProviderOptions &providerOptions = QgsDataProvider::ProviderOptions() );
+    QgsDataProvider( const QString &uri = QString(),
+                     const QgsDataProvider::ProviderOptions &providerOptions = QgsDataProvider::ProviderOptions(),
+                     QgsDataProvider::ReadFlags flags = QgsDataProvider::ReadFlags() );
 
     /**
      * Returns the coordinate system for the data source.
@@ -142,7 +165,7 @@ class CORE_EXPORT QgsDataProvider : public QObject
     {
       if ( expandAuthConfig && mDataSourceURI.contains( QLatin1String( "authcfg" ) ) )
       {
-        QgsDataSourceUri uri( mDataSourceURI );
+        const QgsDataSourceUri uri( mDataSourceURI );
         return uri.uri( expandAuthConfig );
       }
       else
@@ -151,6 +174,16 @@ class CORE_EXPORT QgsDataProvider : public QObject
       }
     }
 
+    /**
+     * Returns a short comment for the data that this provider is
+     * providing access to (e.g. the comment for postgres table).
+     *
+     * \note The default implementation returns an empty string.
+     * \since QGIS 3.14
+     */
+    virtual QString dataComment() const { return QString(); };
+
+
     /**
      * Set the data source specification.
      *
@@ -171,6 +204,24 @@ class CORE_EXPORT QgsDataProvider : public QObject
       return QgsDataSourceUri( mDataSourceURI );
     }
 
+    /**
+     * Returns the provider's temporal capabilities.
+     *
+     * This may be NULLPTR, depending on the data provider.
+     *
+     * \since QGIS 3.14
+     */
+    virtual QgsDataProviderTemporalCapabilities *temporalCapabilities();
+
+    /**
+     * Returns the provider's temporal capabilities.
+     *
+     * This may be NULLPTR, depending on the data provider.
+     *
+     * \since QGIS 3.14
+     */
+    virtual const QgsDataProviderTemporalCapabilities *temporalCapabilities() const SIP_SKIP;
+
     /**
      * Returns the extent of the layer
      * \returns QgsRectangle containing the extent of the layer
@@ -355,11 +406,14 @@ class CORE_EXPORT QgsDataProvider : public QObject
       return QString();
     }
 
+    // TODO QGIS 4 -> Make `reloadData()` non virtual. This should be implemented in `reloadProviderData()`.
+
     /**
-     * Reloads the data from the source by calling reloadProviderData() implemented
-     * by providers with data caches to synchronize, changes in the data source, feature
-     * counts and other specific actions.
+     * Reloads the data from the source for providers with data caches to synchronize,
+     * changes in the data source, feature counts and other specific actions.
      * Emits the `dataChanged` signal
+     *
+     * \note only available for providers which implement the reloadProviderData() method.
      */
     virtual void reloadData();
 
@@ -599,6 +653,9 @@ class CORE_EXPORT QgsDataProvider : public QObject
     //! Sets error message
     void setError( const QgsError &error ) { mError = error;}
 
+    //! Read flags. It's up to the subclass to respect these when needed
+    QgsDataProvider::ReadFlags mReadFlags = QgsDataProvider::ReadFlags();
+
   private:
 
     /**
@@ -607,10 +664,10 @@ class CORE_EXPORT QgsDataProvider : public QObject
      */
     QString mDataSourceURI;
 
-    QMap< int, QVariant > mProviderProperties;
-
     QgsDataProvider::ProviderOptions mOptions;
 
+    QMap< int, QVariant > mProviderProperties;
+
     /**
      * Protects options from being accessed concurrently
      */
@@ -623,5 +680,6 @@ class CORE_EXPORT QgsDataProvider : public QObject
     virtual void reloadProviderData() {}
 };
 
+Q_DECLARE_OPERATORS_FOR_FLAGS( QgsDataProvider::ReadFlags )
 
 #endif
