@@ -17,7 +17,7 @@ email                : sherman at mrcc.com
 #define QGSFEATURE_H
 
 #include "qgis_core.h"
-#include "qgis.h"
+#include "qgis_sip.h"
 
 #include <QExplicitlySharedDataPointer>
 #include <QList>
@@ -29,12 +29,14 @@ email                : sherman at mrcc.com
 
 #include "qgsattributes.h"
 #include "qgsfields.h"
-
+#include "qgsfeatureid.h"
+#include <memory>
 class QgsFeature;
 class QgsFeaturePrivate;
 class QgsField;
 class QgsGeometry;
 class QgsRectangle;
+class QgsAbstractGeometry;
 
 
 /***************************************************************************
@@ -43,19 +45,10 @@ class QgsRectangle;
  * See details in QEP #17
  ****************************************************************************/
 
-// feature id class (currently 64 bit)
-
-// 64 bit feature ids
-typedef qint64 QgsFeatureId SIP_SKIP;
-#define FID_IS_NEW(fid)     (fid<0)
-#define FID_TO_NUMBER(fid)  static_cast<qint64>(fid)
-#define FID_TO_STRING(fid)  QString::number( fid )
-#define STRING_TO_FID(str)  (str).toLongLong()
-
 
 /**
  * \ingroup core
- * The feature class encapsulates a single feature including its id,
+ * \brief The feature class encapsulates a single feature including its id,
  * geometry and a list of field/values attributes.
  * \note QgsFeature objects are implicitly shared.
  */
@@ -71,6 +64,7 @@ class CORE_EXPORT QgsFeature
     Q_PROPERTY( QgsFeatureId id READ id WRITE setId )
     Q_PROPERTY( QgsAttributes attributes READ attributes WRITE setAttributes )
     Q_PROPERTY( QgsFields fields READ fields WRITE setFields )
+    Q_PROPERTY( QgsGeometry geometry READ geometry WRITE setGeometry )
 
   public:
 
@@ -102,7 +96,7 @@ class CORE_EXPORT QgsFeature
     int fieldIdx = sipCpp->fieldNameIndex( *a0 );
     if ( fieldIdx == -1 )
     {
-      PyErr_SetString( PyExc_KeyError, a0->toAscii() );
+      PyErr_SetString( PyExc_KeyError, a0->toLatin1() );
       sipIsErr = 1;
     }
     else
@@ -137,7 +131,7 @@ class CORE_EXPORT QgsFeature
     int fieldIdx = sipCpp->fieldNameIndex( *a0 );
     if ( fieldIdx == -1 )
     {
-      PyErr_SetString( PyExc_KeyError, a0->toAscii() );
+      PyErr_SetString( PyExc_KeyError, a0->toLatin1() );
       sipIsErr = 1;
     }
     else
@@ -169,7 +163,7 @@ class CORE_EXPORT QgsFeature
     int fieldIdx = sipCpp->fieldNameIndex( *a0 );
     if ( fieldIdx == -1 )
     {
-      PyErr_SetString( PyExc_KeyError, a0->toAscii() );
+      PyErr_SetString( PyExc_KeyError, a0->toLatin1() );
       sipIsErr = 1;
     }
     else
@@ -182,9 +176,9 @@ class CORE_EXPORT QgsFeature
      * \param id feature id
      */
 #ifndef SIP_RUN
-    QgsFeature( QgsFeatureId id = QgsFeatureId() );
+    QgsFeature( QgsFeatureId id = FID_NULL );
 #else
-    QgsFeature( qint64 id = 0 );
+    QgsFeature( qint64 id = FID_NULL );
 #endif
 
     /**
@@ -193,9 +187,9 @@ class CORE_EXPORT QgsFeature
      * \param id feature id
      */
 #ifndef SIP_RUN
-    QgsFeature( const QgsFields &fields, QgsFeatureId id = QgsFeatureId() );
+    QgsFeature( const QgsFields &fields, QgsFeatureId id = FID_NULL );
 #else
-    QgsFeature( const QgsFields &fields, qint64 id = 0 );
+    QgsFeature( const QgsFields &fields, qint64 id = FID_NULL );
 #endif
 
     /**
@@ -221,7 +215,7 @@ class CORE_EXPORT QgsFeature
     virtual ~QgsFeature();
 
     /**
-     * Get the feature ID for this feature.
+     * Gets the feature ID for this feature.
      * \returns feature ID
      * \see setId()
      */
@@ -238,11 +232,17 @@ class CORE_EXPORT QgsFeature
      * Returns the feature's attributes.
      * \returns list of feature's attributes
      * \see setAttributes
-     * \since QGIS 2.9
      * \note Alternatively in Python: iterate feature, eg. @code [attr for attr in feature] @endcode
+     * \since QGIS 2.9
      */
     QgsAttributes attributes() const;
 
+    /**
+     * Returns the number of attributes attached to the feature.
+     * \since QGIS 3.18
+     */
+    int attributeCount() const;
+
     /**
      * Sets the feature's attributes.
      * The feature will be valid after.
@@ -257,8 +257,8 @@ class CORE_EXPORT QgsFeature
      * The feature will be valid if it was successful.
      * \param field the index of the field to set
      * \param attr the value of the attribute
-     * \returns false, if the field index does not exist
-     * \note For Python: raises a KeyError exception instead of returning false
+     * \returns FALSE, if the field index does not exist
+     * \note For Python: raises a KeyError exception instead of returning FALSE
      * \note Alternatively in Python: @code feature[field] = attr @endcode
      * \see setAttributes
      */
@@ -291,9 +291,34 @@ class CORE_EXPORT QgsFeature
     /**
      * Initialize this feature with the given number of fields. Discard any previously set attribute data.
      * \param fieldCount Number of fields to initialize
+     *
+     * \see resizeAttributes()
      */
     void initAttributes( int fieldCount );
 
+    /**
+     * Resizes the attributes attached to this feature to the given number of fields.
+     *
+     * If the new \a fieldCount is greater than the original number of fields then the additional attributes will
+     * be filled with NULL values. All existing attributes will remain unchanged.
+     *
+     * If the new \a fieldCount is less than the original number of fields then the unwanted values will be discarded from the
+     * end of the existing attributes.
+     *
+     * \see initAttributes()
+     * \see padAttributes()
+     * \since QGIS 3.18
+     */
+    void resizeAttributes( int fieldCount );
+
+    /**
+     * Resizes the attributes attached to this feature by appending the specified \a count of NULL values to the end of the existing attributes.
+     *
+     * \see resizeAttributes()
+     * \since QGIS 3.18
+     */
+    void padAttributes( int count );
+
     /**
      * Deletes an attribute and its value.
      * \param field the index of the field
@@ -324,13 +349,13 @@ class CORE_EXPORT QgsFeature
 
     /**
      * Sets the validity of the feature.
-     * \param validity set to true if feature is valid
+     * \param validity set to TRUE if feature is valid
      * \see isValid
      */
     void setValid( bool validity );
 
     /**
-     * Returns true if the feature has an associated geometry.
+     * Returns TRUE if the feature has an associated geometry.
      * \see geometry()
      * \since QGIS 3.0.
      */
@@ -352,6 +377,42 @@ class CORE_EXPORT QgsFeature
      */
     void setGeometry( const QgsGeometry &geometry );
 
+    /**
+     * Set the feature's \a geometry. Ownership of the geometry is transferred to the feature.
+     * The feature will be made valid after calling this method.
+     *
+     * This method is a shortcut for calling:
+     * \code{.py}
+     *   feature.setGeometry( QgsGeometry( geometry ) )
+     * \endcode
+     *
+     * ### Example
+     *
+     * \code{.py}
+     *   # Sets a feature's geometry to a point geometry
+     *   feature.setGeometry( QgsPoint( 210, 41 ) )
+     *   print(feature.geometry())
+     *   # output: <QgsGeometry: Point (210 41)>
+     *
+     *   # Sets a feature's geometry to a line string
+     *   feature.setGeometry( QgsLineString( [ QgsPoint( 210, 41 ), QgsPoint( 301, 55 ) ] ) )
+     *   print(feature.geometry())
+     *   # output: <QgsGeometry: LineString (210 41, 301 55)>
+     * \endcode
+     *
+     * \see geometry()
+     * \see clearGeometry()
+     * \since QGIS 3.6
+     */
+#ifndef SIP_RUN
+    void setGeometry( std::unique_ptr< QgsAbstractGeometry > geometry );
+#else
+    void setGeometry( QgsAbstractGeometry *geometry SIP_TRANSFER );
+    % MethodCode
+    sipCpp->setGeometry( std::unique_ptr< QgsAbstractGeometry>( a0 ) );
+    % End
+#endif
+
     /**
      * Removes any geometry associated with the feature.
      * \see setGeometry()
@@ -363,11 +424,11 @@ class CORE_EXPORT QgsFeature
     /**
      * Assign a field map with the feature to allow attribute access by attribute name.
      *  \param fields The attribute fields which this feature holds
-     *  \param initAttributes If true, attributes are initialized. Clears any data previously assigned.
-     *                        C++: Defaults to false
-     *                        Python: Defaults to true
-     * \since QGIS 2.9
+     *  \param initAttributes If TRUE, attributes are initialized. Clears any data previously assigned.
+     *                        C++: Defaults to FALSE
+     *                        Python: Defaults to TRUE
      * \see fields
+     * \since QGIS 2.9
      */
     void setFields( const QgsFields &fields, bool initAttributes = false SIP_PYARGDEFAULT( true ) );
 
@@ -378,13 +439,13 @@ class CORE_EXPORT QgsFeature
     QgsFields fields() const;
 
     /**
-     * Insert a value into attribute. Returns false if attribute name could not be converted to index.
+     * Insert a value into attribute. Returns FALSE if attribute name could not be converted to index.
      *  Field map must be associated using setFields() before this method can be used.
      *  The feature will be valid if it was successful
      *  \param name The name of the field to set
      *  \param value The value to set
-     *  \returns false if attribute name could not be converted to index (C++ only)
-     *  \note For Python: raises a KeyError exception instead of returning false
+     *  \returns FALSE if attribute name could not be converted to index (C++ only)
+     *  \note For Python: raises a KeyError exception instead of returning FALSE
      *  \note Alternatively in Python: @code feature[name] = attr @endcode
      *  \see setFields
      */
@@ -396,7 +457,7 @@ class CORE_EXPORT QgsFeature
     int fieldIdx = sipCpp->fieldNameIndex( *a0 );
     if ( fieldIdx == -1 )
     {
-      PyErr_SetString( PyExc_KeyError, a0->toAscii() );
+      PyErr_SetString( PyExc_KeyError, a0->toLatin1() );
       sipIsErr = 1;
     }
     else
@@ -417,8 +478,8 @@ class CORE_EXPORT QgsFeature
      * Removes an attribute value by field name. Field map must be associated using setFields()
      *  before this method can be used.
      *  \param name The name of the field to delete
-     *  \returns false if attribute name could not be converted to index (C++ only)
-     *  \note For Python: raises a KeyError exception instead of returning false
+     *  \returns FALSE if attribute name could not be converted to index (C++ only)
+     *  \note For Python: raises a KeyError exception instead of returning FALSE
      *  \note Alternatively in Python: @code del feature[name] @endcode
      *  \see setFields
      */
@@ -428,7 +489,7 @@ class CORE_EXPORT QgsFeature
     int fieldIdx = sipCpp->fieldNameIndex( *a0 );
     if ( fieldIdx == -1 )
     {
-      PyErr_SetString( PyExc_KeyError, a0->toAscii() );
+      PyErr_SetString( PyExc_KeyError, a0->toLatin1() );
       sipIsErr = 1;
       sipRes = false;
     }
@@ -457,7 +518,7 @@ class CORE_EXPORT QgsFeature
     int fieldIdx = sipCpp->fieldNameIndex( *a0 );
     if ( fieldIdx == -1 )
     {
-      PyErr_SetString( PyExc_KeyError, a0->toAscii() );
+      PyErr_SetString( PyExc_KeyError, a0->toLatin1() );
       sipIsErr = 1;
     }
     else
@@ -506,6 +567,17 @@ class CORE_EXPORT QgsFeature
      */
     int fieldNameIndex( const QString &fieldName ) const;
 
+    /**
+     * Returns the approximate RAM usage of the feature, in bytes.
+     *
+     * This method takes into account the size of variable elements (strings,
+     * geometry, ...), but the value returned should be considered as a lower
+     * bound estimation.
+     *
+     * \since QGIS 3.16
+     */
+    int approximateMemoryUsage() const;
+
     //! Allows direct construction of QVariants from features.
     operator QVariant() const
     {
@@ -539,13 +611,6 @@ typedef QMap<QgsFeatureId, QgsGeometry> QgsGeometryMap;
 typedef QMap<qint64, QgsGeometry> QgsGeometryMap;
 #endif
 
-
-#ifndef SIP_RUN
-typedef QSet<QgsFeatureId> QgsFeatureIds;
-#else
-typedef QSet<qint64> QgsFeatureIds;
-#endif
-
 typedef QList<QgsFeature> QgsFeatureList;
 
 uint qHash( const QgsFeature &key, uint seed = 0 )  SIP_SKIP;
