@@ -20,10 +20,11 @@
 
 #include "qgis_core.h"
 #include "qgis_sip.h"
-#include "qgis.h"
 #include "qgslayoutitemmapitem.h"
 #include "qgssymbol.h"
+#include "qgstextformat.h"
 #include <QPainter>
+#include <QVector2D>
 
 class QgsCoordinateTransform;
 class QgsLayoutItemMapGrid;
@@ -150,7 +151,8 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
     {
       MapUnit, //!< Grid units follow map units
       MM, //!< Grid units in millimeters
-      CM //!< Grid units in centimeters
+      CM, //!< Grid units in centimeters
+      DynamicPageSizeBased, //!< Dynamically sized, based on a on-page size range
     };
 
     /**
@@ -192,7 +194,10 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
       Horizontal = 0, //!< Draw annotations horizontally
       Vertical, //!< Draw annotations vertically, ascending
       VerticalDescending, //!< Draw annotations vertically, descending
-      BoundaryDirection //!< Annotations follow the boundary direction
+      BoundaryDirection, //!< Annotations follow the boundary direction
+      AboveTick, //!< Draw annotations parallel to tick (above the line)
+      OnTick, //!< Draw annotations parallel to tick (on the line)
+      UnderTick, //!< Draw annotations parallel to tick (under the line)
     };
 
     /**
@@ -219,7 +224,7 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
       Left, //!< Left border
       Right, //!< Right border
       Bottom, //!< Bottom border
-      Top //!< Top border
+      Top, //!< Top border
     };
 
     /**
@@ -232,7 +237,18 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
       InteriorTicks,  //!< Tick markers drawn inside map frame
       ExteriorTicks,  //!< Tick markers drawn outside map frame
       InteriorExteriorTicks, //!< Tick markers drawn both inside and outside the map frame
-      LineBorder //!< Simple solid line frame
+      LineBorder, //!< Simple solid line frame
+      LineBorderNautical, //!< Simple solid line frame, with nautical style diagonals on corners
+      ZebraNautical, //!< Black/white pattern, with nautical style diagonals on corners
+    };
+
+    /**
+     * Tick length mode (useful for rotated grids)
+     */
+    enum TickLengthMode
+    {
+      OrthogonalTicks = 0, //!< Align ticks orthogonaly
+      NormalizedTicks, //!< Constant tick lengths
     };
 
     /**
@@ -307,6 +323,8 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
      */
     void calculateMaxExtension( double &top, double &right, double &bottom, double &left ) const;
 
+    void setEnabled( bool enabled ) override;
+
     //
     // GRID UNITS
     //
@@ -389,6 +407,50 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
      */
     double offsetY() const { return mGridOffsetY; }
 
+    /**
+     * Returns the minimum width (in millimeters) for grid segments. This
+     * property is only effective if the units() is set
+     * to DynamicPageSizeBased.
+     * \see units()
+     * \see setMinimumIntervalWidth()
+     * \see maximumIntervalWidth()
+     * \since QGIS 3.10
+     */
+    double minimumIntervalWidth() const { return mMinimumIntervalWidth; }
+
+    /**
+     * Sets the minimum \a width (in millimeters) for grid segments. This
+     * property is only effective if the units() is set
+     * to DynamicPageSizeBased.
+     * \see minimumIntervalWidth()
+     * \see setMaximumIntervalWidth()
+     * \see setUnits()
+     * \since QGIS 3.10
+     */
+    void setMinimumIntervalWidth( double width );
+
+    /**
+     * Returns the maximum width (in millimeters) for grid segments. This
+     * property is only effective if the units() is set
+     * to DynamicPageSizeBased.
+     * \see units()
+     * \see setMaximumIntervalWidth()
+     * \see minimumIntervalWidth()
+     * \since QGIS 3.10
+     */
+    double maximumIntervalWidth() const { return mMaximumIntervalWidth; }
+
+    /**
+     * Sets the maximum \a width (in millimeters) for grid segments. This
+     * property is only effective if the units() is set
+     * to DynamicPageSizeBased.
+     * \see maximumIntervalWidth()
+     * \see setMinimumIntervalWidth()
+     * \see setUnits()
+     * \since QGIS 3.10
+     */
+    void setMaximumIntervalWidth( double width );
+
     //
     // GRID APPEARANCE
     //
@@ -412,7 +474,7 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
      * with QgsLayoutItemMapGrid::Cross styles.
      * \see crossLength()
      */
-    void setCrossLength( const double length ) { mCrossLength = length; }
+    void setCrossLength( const double length );
 
     /**
      * Retrieves the length (in layout units) of the cross segments drawn for the grid. This is only used for grids
@@ -514,28 +576,52 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
     bool annotationEnabled() const { return mShowGridAnnotation; }
 
     /**
-     * Sets the \a font used for drawing grid annotations.
-     * \see annotationFont()
+     * Sets the text \a format to use when rendering grid annotations.
+     *
+     * \see annotationTextFormat()
+     * \since QGIS 3.16
      */
-    void setAnnotationFont( const QFont &font ) { mGridAnnotationFont = font; }
+    void setAnnotationTextFormat( const QgsTextFormat &format ) { mAnnotationFormat = format; }
+
+    /**
+     * Returns the text format used when rendering grid annotations.
+     *
+     * \see setAnnotationTextFormat()
+     * \since QGIS 3.16
+     */
+    QgsTextFormat annotationTextFormat() const { return mAnnotationFormat; }
+
+    /**
+     * Sets the \a font used for drawing grid annotations.
+     * Shortcut for annotationTextFormat().setFont().
+     * \see annotationFont()
+     * \deprecated use setAnnotationTextFormat() instead
+     */
+    Q_DECL_DEPRECATED void setAnnotationFont( const QFont &font ) SIP_DEPRECATED;
 
     /**
      * Returns the font used for drawing grid annotations.
+     * Shortcut for annotationTextFormat().font().
      * \see setAnnotationFont()
+     * \deprecated use annotationTextFormat() instead
      */
-    QFont annotationFont() const { return mGridAnnotationFont; }
+    Q_DECL_DEPRECATED QFont annotationFont() const SIP_DEPRECATED;
 
     /**
      * Sets the font \a color used for drawing grid annotations.
+     * Shortcut for annotationTextFormat().setColor() and annotationTextFormat().setOpacity().
      * \see annotationFontColor()
+     * \deprecated use setAnnotationTextFormat() instead
      */
-    void setAnnotationFontColor( const QColor &color ) { mGridAnnotationFontColor = color; }
+    Q_DECL_DEPRECATED void setAnnotationFontColor( const QColor &color ) SIP_DEPRECATED;
 
     /**
      * Returns the font color used for drawing grid annotations.
+     * Shortcut for annotationTextFormat().color() and annotationTextFormat().opacity().
      * \see setAnnotationFontColor()
+     * \deprecated use annotationTextFormat() instead
      */
-    QColor annotationFontColor() const { return mGridAnnotationFontColor; }
+    Q_DECL_DEPRECATED QColor annotationFontColor() const SIP_DEPRECATED;
 
     /**
      * Sets the coordinate \a precision for grid annotations.
@@ -588,7 +674,7 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
      * Sets the \a distance between the map frame and annotations. Units are layout units.
      * \see annotationFrameDistance()
      */
-    void setAnnotationFrameDistance( const double distance ) { mAnnotationFrameDistance = distance; }
+    void setAnnotationFrameDistance( const double distance );
 
     /**
      * Returns the distance between the map frame and annotations. Units are in layout units.
@@ -681,7 +767,7 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
     /**
      * Sets whether the grid frame is drawn for a certain side of the map item.
      * \param flag flag for grid frame side
-     * \param on set to true to draw grid frame on that side of the map
+     * \param on set to TRUE to draw grid frame on that side of the map
      * \see setFrameSideFlags()
      * \see frameSideFlags()
      * \see testFrameSideFlag()
@@ -701,7 +787,7 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
      * Tests whether the grid frame should be drawn on a specified side of the map
      * item.
      * \param flag flag for grid frame side
-     * \returns true if grid frame should be drawn for that side of the map
+     * \returns TRUE if grid frame should be drawn for that side of the map
      * \see setFrameSideFlags()
      * \see setFrameSideFlag()
      * \see frameSideFlags()
@@ -714,7 +800,7 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
      * setFramePenSize method.
      * \see frameWidth()
      */
-    void setFrameWidth( const double width ) { mGridFrameWidth = width; }
+    void setFrameWidth( const double width );
 
     /**
      * Gets the grid frame width in layout units. This property controls how wide the grid frame is.
@@ -724,12 +810,140 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
      */
     double frameWidth() const { return mGridFrameWidth; }
 
+    /**
+     * Enable/disable ticks rotation for rotated or reprojected grids.
+     * \see rotatedTicksEnabled()
+     * \since QGIS 3.16
+     */
+    void setRotatedTicksEnabled( const bool state ) { mRotatedTicksEnabled = state; }
+
+    /**
+     * Gets whether ticks rotation for rotated or reprojected grids is enabled.
+     * \see setRotatedTicksEnabled()
+     * \since QGIS 3.16
+     */
+    double rotatedTicksEnabled() const { return mRotatedTicksEnabled; }
+
+    /**
+     * Sets the tick length calculation mode.
+     * \see rotatedTicksLengthMode()
+     * \since QGIS 3.16
+     */
+    void setRotatedTicksLengthMode( const TickLengthMode mode ) { mRotatedTicksLengthMode = mode; }
+
+    /**
+     * Returns the grid frame style.
+     * \see setRotatedTicksLengthMode()
+     * \since QGIS 3.16
+     */
+    TickLengthMode rotatedTicksLengthMode() const { return mRotatedTicksLengthMode; }
+
+    /**
+     * Sets the \a minimum angle (in degrees) below which ticks are not drawn.
+     * \see rotatedTicksMinimumAngle()
+     * \since QGIS 3.16
+     */
+    void setRotatedTicksMinimumAngle( const double angle ) { mRotatedTicksMinimumAngle = angle; }
+
+    /**
+     * Gets the \a minimum angle (in degrees) below which ticks are not drawn.
+     * \see setRotatedTicksMinimumAngle()
+     * \since QGIS 3.16
+     */
+    double rotatedTicksMinimumAngle() const { return mRotatedTicksMinimumAngle; }
+
+    /**
+     * Sets the \a margin to corners (in canvas units) below which outwards facing ticks are not drawn.
+     * \see rotatedTicksMarginToCorner()
+     * \since QGIS 3.16
+     */
+    void setRotatedTicksMarginToCorner( const double margin ) { mRotatedTicksMarginToCorner = margin; }
+
+    /**
+     * Gets the \a margin to corners (in canvas units) below which outwards facing ticks are not drawn.
+     * \see setRotatedTicksMarginToCorner()
+     * \since QGIS 3.16
+     */
+    double rotatedTicksMarginToCorner() const { return mRotatedTicksMarginToCorner; }
+
+    /**
+     * Enable/disable annotations rotation for rotated or reprojected grids.
+     * \see rotatedAnnotationsEnabled()
+     * \since QGIS 3.16
+     */
+    void setRotatedAnnotationsEnabled( const bool state ) { mRotatedAnnotationsEnabled = state; }
+
+    /**
+     * Gets whether annotations rotation for rotated or reprojected grids is enabled.
+     * \see setRotatedAnnotationsEnabled()
+     * \since QGIS 3.16
+     */
+    double rotatedAnnotationsEnabled() const { return mRotatedAnnotationsEnabled; }
+
+    /**
+     * Sets the annotation length calculation mode.
+     * \see rotatedAnnotationsLengthMode()
+     * \since QGIS 3.16
+    */
+    void setRotatedAnnotationsLengthMode( const TickLengthMode mode ) { mRotatedAnnotationsLengthMode = mode; }
+
+    /**
+     * Returns the grid frame style.
+     * \see setRotatedAnnotationsLengthMode()
+     * \since QGIS 3.16
+     */
+    TickLengthMode rotatedAnnotationsLengthMode() const { return mRotatedAnnotationsLengthMode; }
+
+    /**
+     * Sets the \a minimum angle (in degrees) below which annotated are not drawn.
+     * \see rotatedAnnotationsMinimumAngle()
+     * \since QGIS 3.16
+     */
+    void setRotatedAnnotationsMinimumAngle( const double angle ) { mRotatedAnnotationsMinimumAngle = angle; }
+
+    /**
+     * Gets the \a minimum angle (in degrees) below which annotated are not drawn.
+     * \see setRotatedAnnotationsMinimumAngle()
+     * \since QGIS 3.16
+     */
+    double rotatedAnnotationsMinimumAngle() const { return mRotatedAnnotationsMinimumAngle; }
+
+    /**
+     * Sets the \a margin to corners (in canvas units) below which outwards facing ticks are not drawn.
+     * \see rotatedAnnotationsMarginToCorner()
+     * \since QGIS 3.16
+     */
+    void setRotatedAnnotationsMarginToCorner( const double margin ) { mRotatedAnnotationsMarginToCorner = margin; }
+
+    /**
+     * Gets the \a margin to corners (in canvas units) below which outwards facing ticks are not drawn.
+     * \see setRotatedAnnotationsMarginToCorner()
+     * \since QGIS 3.16
+     */
+    double rotatedAnnotationsMarginToCorner() const { return mRotatedAnnotationsMarginToCorner; }
+
+    /**
+     * Sets the grid frame margin (in layout units).
+     * This property controls distance between the map frame and the grid frame.
+     * \see frameMargin()
+     * \since QGIS 3.6
+     */
+    void setFrameMargin( const double margin );
+
+    /**
+     * Sets the grid frame Margin (in layout units).
+     * This property controls distance between the map frame and the grid frame.
+     * \see setFrameMargin()
+     * \since QGIS 3.6
+     */
+    double frameMargin() const { return mGridFrameMargin; }
+
     /**
      * Sets the \a width of the stroke drawn in the grid frame.
      * \see framePenSize()
      * \see setFramePenColor()
      */
-    void setFramePenSize( const double width ) { mGridFramePenThickness = width; }
+    void setFramePenSize( const double width );
 
     /**
      * Retrieves the width of the stroke drawn in the grid frame.
@@ -789,6 +1003,17 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
     QColor frameFillColor2() const { return mGridFrameFillColor2; }
 
     QgsExpressionContext createExpressionContext() const override;
+    bool accept( QgsStyleEntityVisitorInterface *visitor ) const override;
+    void refresh() override;
+
+  signals:
+
+    /**
+     * Emitted whenever the grid's CRS is changed.
+     *
+     * \since QGIS 3.18
+     */
+    void crsChanged();
 
   private:
 
@@ -801,8 +1026,63 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
       double right = 0.0;
       double bottom = 0.0;
       double left = 0.0;
+
+      /**
+       * Updates the specified border of the extension
+       */
+      void UpdateBorder( BorderSide border, double value )
+      {
+        switch ( border )
+        {
+          case QgsLayoutItemMapGrid::Left:
+            left = std::max( left, value );
+            break;
+          case QgsLayoutItemMapGrid::Right:
+            right = std::max( right, value );
+            break;
+          case QgsLayoutItemMapGrid::Top:
+            top = std::max( top, value );
+            break;
+          case QgsLayoutItemMapGrid::Bottom:
+            bottom = std::max( bottom, value );
+            break;
+        }
+      }
+
+      /**
+       * Updates all borders of the extension
+       */
+      void UpdateAll( double value )
+      {
+        left = std::max( left, value );
+        right = std::max( right, value );
+        top = std::max( top, value );
+        bottom = std::max( bottom, value );
+      }
     };
 
+    struct GridLineAnnotation
+    {
+      BorderSide border = Left; // border on which the annotation is
+      QVector2D position; // position on the frame
+      QVector2D vector; // vector towards map center
+      double angle = 0; // the (acute) angle formed between the vector and the border
+    };
+
+    /**
+     * Helper that represents a grid line, for drawing the line itself an the
+     * anotations on the frame.
+     */
+    struct GridLine
+    {
+      QPolygonF line; // the actual line, can be straight with two points or curved if transformed
+      double coordinate; // the coordinate value
+      QgsLayoutItemMapGrid::AnnotationCoordinate coordinateType; // whether it's a latitude or longitude line
+      GridLineAnnotation startAnnotation; // the annotation on the start point
+      GridLineAnnotation endAnnotation; // the annotation on the end point
+    };
+    mutable QList< GridLine > mGridLines;
+
     //! True if a re-transformation of grid lines is required
     mutable bool mTransformDirty = true;
 
@@ -816,10 +1096,10 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
     double mGridOffsetX = 0.0;
     //! Grid line offset in y-direction
     double mGridOffsetY = 0.0;
-    //! Font for grid line annotation
-    QFont mGridAnnotationFont;
-    //! Font color for grid coordinates
-    QColor mGridAnnotationFontColor  = Qt::black;
+
+    //! Text format for grid annotations
+    QgsTextFormat mAnnotationFormat;
+
     //! Digits after the dot
     int mGridAnnotationPrecision = 3;
     //! True if coordinate values should be drawn
@@ -860,6 +1140,7 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
     mutable std::unique_ptr< QgsExpression > mGridAnnotationExpression;
 
     FrameStyle mGridFrameStyle = QgsLayoutItemMapGrid::NoFrame;
+
     FrameSideFlags mGridFrameSides;
     double mGridFrameWidth = 2.0;
     double mGridFramePenThickness = 0.3;
@@ -867,6 +1148,18 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
     QColor mGridFrameFillColor1 = Qt::white;
     QColor mGridFrameFillColor2 = Qt::black;
     double mCrossLength = 3.0;
+    double mGridFrameMargin = 0.0;
+    bool mRotatedTicksEnabled = false;
+    TickLengthMode mRotatedTicksLengthMode = QgsLayoutItemMapGrid::OrthogonalTicks;
+    double mRotatedTicksMinimumAngle = 0.0;
+    double mRotatedTicksMarginToCorner = 0.0;
+    bool mRotatedAnnotationsEnabled = false;
+    TickLengthMode mRotatedAnnotationsLengthMode = QgsLayoutItemMapGrid::OrthogonalTicks;
+    double mRotatedAnnotationsMinimumAngle = 0.0;
+    double mRotatedAnnotationsMarginToCorner = 0.0;
+
+    double mMinimumIntervalWidth = 50;
+    double mMaximumIntervalWidth = 100;
 
     //! Divisions for frame on left map side
     DisplayMode mLeftFrameDivisions = QgsLayoutItemMapGrid::ShowAll;
@@ -892,74 +1185,78 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
     QRectF mPrevPaintRect;
     mutable QPolygonF mPrevMapPolygon;
 
-    class QgsMapAnnotation
-    {
-      public:
-        double coordinate;
-        QPointF itemPosition;
-        QgsLayoutItemMapGrid::AnnotationCoordinate coordinateType;
-    };
+    bool mEvaluatedEnabled = true;
+    double mEvaluatedIntervalX = 0;
+    double mEvaluatedIntervalY = 0;
+    double mEvaluatedOffsetX = 0;
+    double mEvaluatedOffsetY = 0;
+    double mEvaluatedGridFrameWidth = 0;
+    double mEvaluatedGridFrameMargin = 0;
+    double mEvaluatedAnnotationFrameDistance = 0;
+    double mEvaluatedCrossLength = 0;
+    double mEvaluatedGridFrameLineThickness = 0;
+    DisplayMode mEvaluatedLeftGridAnnotationDisplay = QgsLayoutItemMapGrid::ShowAll;
+    DisplayMode mEvaluatedRightGridAnnotationDisplay = QgsLayoutItemMapGrid::ShowAll;
+    DisplayMode mEvaluatedTopGridAnnotationDisplay = QgsLayoutItemMapGrid::ShowAll;
+    DisplayMode mEvaluatedBottomGridAnnotationDisplay = QgsLayoutItemMapGrid::ShowAll;
+    DisplayMode mEvaluatedLeftFrameDivisions = QgsLayoutItemMapGrid::ShowAll;
+    DisplayMode mEvaluatedRightFrameDivisions = QgsLayoutItemMapGrid::ShowAll;
+    DisplayMode mEvaluatedTopFrameDivisions = QgsLayoutItemMapGrid::ShowAll;
+    DisplayMode mEvaluatedBottomFrameDivisions = QgsLayoutItemMapGrid::ShowAll;
+
+    /**
+     * Updates the grid lines annotation positions
+     */
+    void updateGridLinesAnnotationsPositions() const;
 
     /**
      * Draws the map grid. If extension is specified, then no grid will be drawn and instead the maximum extension
      * for the grid outside of the map frame will be calculated.
      */
-    void drawGridFrame( QPainter *p, const QList< QPair< double, QLineF > > &hLines, const QList< QPair< double, QLineF > > &vLines, GridExtension *extension = nullptr ) const;
+    void drawGridFrame( QPainter *p, GridExtension *extension = nullptr ) const;
 
     /**
      * Draw coordinates for mGridAnnotationType Coordinate
-        \param p drawing painter
-        \param hLines horizontal coordinate lines in item coordinates
-        \param vLines vertical coordinate lines in item coordinates
-        \param expressionContext expression context for evaluating custom annotation formats
-        \param extension optional. If specified, nothing will be drawn and instead the maximum extension for the grid
-        annotations will be stored in this variable.
+     * \param context destination render context
+     * \param expressionContext expression context for evaluating custom annotation formats
+     * \param extension optional. If specified, nothing will be drawn and instead the maximum extension for the grid
+     * annotations will be stored in this variable.
      */
-    void drawCoordinateAnnotations( QPainter *p, const QList< QPair< double, QLineF > > &hLines, const QList< QPair< double, QLineF > > &vLines, QgsExpressionContext &expressionContext, GridExtension *extension = nullptr ) const;
+    void drawCoordinateAnnotations( QgsRenderContext &context, QgsExpressionContext &expressionContext, GridExtension *extension = nullptr ) const;
 
     /**
      * Draw an annotation. If optional extension argument is specified, nothing will be drawn and instead
      * the extension of the annotation outside of the map frame will be stored in this variable.
      */
-    void drawCoordinateAnnotation( QPainter *p, QPointF pos, const QString &annotationString, AnnotationCoordinate coordinateType, GridExtension *extension = nullptr ) const;
-
-    /**
-     * Draws a single annotation
-     * \param p drawing painter
-     * \param pos item coordinates where to draw
-     * \param rotation text rotation
-     * \param annotationText the text to draw
-     */
-    void drawAnnotation( QPainter *p, QPointF pos, int rotation, const QString &annotationText ) const;
+    void drawCoordinateAnnotation( QgsRenderContext &context, GridLineAnnotation annot, const QString &annotationString, AnnotationCoordinate coordinateType, GridExtension *extension = nullptr ) const;
 
     QString gridAnnotationString( double value, AnnotationCoordinate coord, QgsExpressionContext &expressionContext ) const;
 
     /**
-     * Returns the grid lines with associated coordinate value
-        \returns 0 in case of success*/
-    int xGridLines( QList< QPair< double, QLineF > > &lines ) const;
+     * Computes the grid lines with associated coordinate value
+     * \returns 0 in case of success
+    */
+    int xGridLines() const;
 
     /**
-     * Returns the grid lines for the y-coordinates. Not vertical in case of rotation
-        \returns 0 in case of success*/
-    int yGridLines( QList< QPair< double, QLineF > > &lines ) const;
+     * Computes the grid lines for the y-coordinates. Not vertical in case of rotation
+     * \returns 0 in case of success
+    */
+    int yGridLines() const;
 
-    int xGridLinesCrsTransform( const QgsRectangle &bbox, const QgsCoordinateTransform &t, QList< QPair< double, QPolygonF > > &lines ) const;
+    int xGridLinesCrsTransform( const QgsRectangle &bbox, const QgsCoordinateTransform &t ) const;
 
-    int yGridLinesCrsTransform( const QgsRectangle &bbox, const QgsCoordinateTransform &t, QList< QPair< double, QPolygonF > > &lines ) const;
+    int yGridLinesCrsTransform( const QgsRectangle &bbox, const QgsCoordinateTransform &t ) const;
 
     void drawGridLine( const QLineF &line, QgsRenderContext &context ) const;
 
     void drawGridLine( const QPolygonF &line, QgsRenderContext &context ) const;
 
-    void sortGridLinesOnBorders( const QList< QPair< double, QLineF > > &hLines, const QList< QPair< double, QLineF > > &vLines, QMap< double, double > &leftFrameEntries,
-                                 QMap< double, double > &rightFrameEntries, QMap< double, double > &topFrameEntries, QMap< double, double > &bottomFrameEntries ) const;
-
     /**
      * Draw the grid frame's border. If optional extension argument is specified, nothing will be drawn and instead
      * the maximum extension of the frame border outside of the map frame will be stored in this variable.
      */
-    void drawGridFrameBorder( QPainter *p, const QMap< double, double > &borderPos, BorderSide border, double *extension = nullptr ) const;
+    void drawGridFrameBorder( QPainter *p, BorderSide border, double *extension = nullptr ) const;
 
     /**
      * Returns the item border of a point (in item coordinates)
@@ -976,10 +1273,9 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
     QPolygonF scalePolygon( const QPolygonF &polygon, double scale ) const;
 
     //! Draws grid if CRS is different to map CRS
-    void drawGridCrsTransform( QgsRenderContext &context, double dotsPerMM, QList< QPair< double, QLineF > > &horizontalLines,
-                               QList< QPair< double, QLineF > > &verticalLines, bool calculateLinesOnly = false ) const;
+    void drawGridCrsTransform( QgsRenderContext &context, double dotsPerMM, bool calculateLinesOnly = false ) const;
 
-    void drawGridNoTransform( QgsRenderContext &context, double dotsPerMM, QList<QPair<double, QLineF> > &horizontalLines, QList<QPair<double, QLineF> > &verticalLines, bool calculateLinesOnly = false ) const;
+    void drawGridNoTransform( QgsRenderContext &context, double dotsPerMM, bool calculateLinesOnly = false ) const;
 
     void createDefaultGridLineSymbol();
 
@@ -987,16 +1283,23 @@ class CORE_EXPORT QgsLayoutItemMapGrid : public QgsLayoutItemMapItem
 
     void drawGridMarker( QPointF point, QgsRenderContext &context ) const;
 
-    void drawGridFrameZebraBorder( QPainter *p, const QMap<double, double> &borderPos, BorderSide border, double *extension = nullptr ) const;
+    void drawGridFrameZebra( QPainter *p, GridExtension *extension = nullptr ) const;
 
-    void drawGridFrameTicks( QPainter *p, const QMap<double, double> &borderPos, BorderSide border, double *extension = nullptr ) const;
+    void drawGridFrameZebraBorder( QPainter *p, BorderSide border, double *extension = nullptr ) const;
 
-    void drawGridFrameLineBorder( QPainter *p, BorderSide border, double *extension = nullptr ) const;
+    void drawGridFrameTicks( QPainter *p, GridExtension *extension = nullptr ) const;
+
+    void drawGridFrameLine( QPainter *p, GridExtension *extension = nullptr ) const;
 
     void calculateCrsTransformLines() const;
 
     bool shouldShowDivisionForSide( AnnotationCoordinate coordinate, BorderSide side ) const;
-    bool shouldShowDivisionForDisplayMode( AnnotationCoordinate coordinate, DisplayMode mode ) const;
+    bool shouldShowAnnotationForSide( AnnotationCoordinate coordinate, BorderSide side ) const;
+    bool shouldShowForDisplayMode( AnnotationCoordinate coordinate, DisplayMode mode ) const;
+    void refreshDataDefinedProperties();
+
+    //! Returns diagonal of map in CRS units
+    double mapWidth() const;
 
     friend class TestQgsLayoutMapGrid;
 
