@@ -34,7 +34,7 @@
 /**
  * \class QgsLocator
  * \ingroup core
- * Handles the management of QgsLocatorFilter objects and async collection of search results from them.
+ * \brief Handles the management of QgsLocatorFilter objects and async collection of search results from them.
  *
  * QgsLocator acts as both a registry for QgsLocatorFilter objects and a means of firing up
  * asynchronous queries against these filter objects.
@@ -145,6 +145,13 @@ class CORE_EXPORT QgsLocator : public QObject
      */
     void clearPreviousResults();
 
+    /**
+     * Returns the list for auto completion
+     * This list is updated when preparing the search
+     * \since QGIS 3.16
+     */
+    QStringList completionList() const {return mAutocompletionList;}
+
   signals:
 
     /**
@@ -153,6 +160,13 @@ class CORE_EXPORT QgsLocator : public QObject
      */
     void foundResult( const QgsLocatorResult &result );
 
+    /**
+     * Emitted when locator has prepared the search (\see QgsLocatorFilter::prepare)
+     * before the search is actually performed
+     * \since QGIS 3.16
+     */
+    void searchPrepared();
+
     /**
      * Emitted when locator has finished a query, either as a result
      * of successful completion or early cancellation.
@@ -171,6 +185,8 @@ class CORE_EXPORT QgsLocator : public QObject
     QList< QgsLocatorFilter * > mFilters;
     QList< QThread * > mActiveThreads;
 
+    QStringList mAutocompletionList;
+
     void cancelRunningQuery();
 
 };
