@@ -48,6 +48,7 @@ class QgsMapLayerStyleManager;
 class QgsProject;
 class QgsStyleEntityVisitorInterface;
 class QgsMapLayerTemporalProperties;
+class QgsMapLayerElevationProperties;
 
 class QDomDocument;
 class QKeyEvent;
@@ -61,7 +62,7 @@ class QPainter;
 
 /**
  * \ingroup core
- * Types of layers that can be added to a map
+ * \brief Types of layers that can be added to a map
  * \since QGIS 3.8
  */
 enum class QgsMapLayerType SIP_MONKEYPATCH_SCOPEENUM_UNNEST( QgsMapLayer, LayerType ) : int
@@ -72,11 +73,12 @@ enum class QgsMapLayerType SIP_MONKEYPATCH_SCOPEENUM_UNNEST( QgsMapLayer, LayerT
   MeshLayer,      //!< Added in 3.2
   VectorTileLayer, //!< Added in 3.14
   AnnotationLayer, //!< Contains freeform, georeferenced annotations. Added in QGIS 3.16
+  PointCloudLayer, //!< Added in 3.18
 };
 
 /**
  * \ingroup core
- * Base class for all map layer types.
+ * \brief Base class for all map layer types.
  * This is the base class for all map layer types (vector, raster).
  */
 class CORE_EXPORT QgsMapLayer : public QObject
@@ -89,6 +91,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     Q_PROPERTY( QgsCoordinateReferenceSystem crs READ crs WRITE setCrs NOTIFY crsChanged )
     Q_PROPERTY( QgsMapLayerType type READ type CONSTANT )
     Q_PROPERTY( bool isValid READ isValid NOTIFY isValidChanged )
+    Q_PROPERTY( double opacity READ opacity WRITE setOpacity NOTIFY opacityChanged )
 
 #ifdef SIP_RUN
     SIP_CONVERT_TO_SUBCLASS_CODE
@@ -118,6 +121,9 @@ class CORE_EXPORT QgsMapLayer : public QObject
         case QgsMapLayerType::AnnotationLayer:
           sipType = sipType_QgsAnnotationLayer;
           break;
+        case QgsMapLayerType::PointCloudLayer:
+          sipType = sipType_QgsPointCloudLayer;
+          break;
         default:
           sipType = nullptr;
           break;
@@ -148,6 +154,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
       Identifiable = 1 << 0, //!< If the layer is identifiable using the identify map tool and as a WMS layer.
       Removable = 1 << 1,    //!< If the layer can be removed from the project. The layer will not be removable from the legend menu entry but can still be removed with an API call.
       Searchable = 1 << 2,   //!< Only for vector-layer, determines if the layer is used in the 'search all layers' locator.
+      Private = 1 << 3,       //!< Determines if the layer is meant to be exposed to the GUI, i.e. visible in the layer legend tree.
     };
     Q_ENUM( LayerFlag )
     Q_DECLARE_FLAGS( LayerFlags, LayerFlag )
@@ -173,10 +180,11 @@ class CORE_EXPORT QgsMapLayer : public QObject
       CustomProperties   = 1 << 11, //!< Custom properties (by plugins for instance)
       GeometryOptions    = 1 << 12, //!< Geometry validation configuration
       Relations          = 1 << 13, //!< Relations
-      Temporal           = 1 << 14, //!< Temporal properties
+      Temporal           = 1 << 14, //!< Temporal properties (since QGIS 3.14)
       Legend             = 1 << 15, //!< Legend settings (since QGIS 3.16)
+      Elevation          = 1 << 16, //!< Elevation settings (since QGIS 3.18)
       AllStyleCategories = LayerConfiguration | Symbology | Symbology3D | Labeling | Fields | Forms | Actions |
-                           MapTips | Diagrams | AttributeTable | Rendering | CustomProperties | GeometryOptions | Relations | Temporal | Legend,
+                           MapTips | Diagrams | AttributeTable | Rendering | CustomProperties | GeometryOptions | Relations | Temporal | Legend | Elevation,
     };
     Q_ENUM( StyleCategory )
     Q_DECLARE_FLAGS( StyleCategories, StyleCategory )
@@ -254,7 +262,7 @@ class CORE_EXPORT QgsMapLayer : public QObject
     /**
      * Returns the layer's data provider, it may be NULLPTR.
      */
-    virtual QgsDataProvider *dataProvider();
+    Q_INVOKABLE virtual QgsDataProvider *dataProvider();
 
     /**
      * Returns the layer's data provider in a const-correct manner, it may be NULLPTR.
@@ -469,13 +477,33 @@ class CORE_EXPORT QgsMapLayer : public QObject
     */
     QPainter::CompositionMode blendMode() const;
 
+    /**
+     * Sets the \a opacity for the layer, where \a opacity is a value between 0 (totally transparent)
+     * and 1.0 (fully opaque).
+     * \see opacity()
+     * \see opacityChanged()
+     * \note Prior to QGIS 3.18, this method was available for vector layers only
+     * \since QGIS 3.18
+     */
+    virtual void setOpacity( double opacity );
+
+    /**
+     * Returns the opacity for the layer, where opacity is a value between 0 (totally transparent)
+     * and 1.0 (fully opaque).
+     * \see setOpacity()
+     * \see opacityChanged()
+     * \note Prior to QGIS 3.18, this method was available for vector layers only
+     * \since QGIS 3.18
+     */
+    virtual double opacity() const;
+
     //! Returns if this layer is read only.
     bool readOnly() const { return isReadOnly(); }
 
     /**
      * Synchronises with changes in the datasource
-        */
-    virtual void reload() {}
+     */
+    Q_INVOKABLE virtual void reload() {}
 
     /**
      * Returns new instance of QgsMapLayerRenderer that will be used for rendering of given context
@@ -1202,6 +1230,13 @@ class CORE_EXPORT QgsMapLayer : public QObject
      */
     virtual QgsMapLayerTemporalProperties *temporalProperties() { return nullptr; }
 
+    /**
+     * Returns the layer's elevation properties. This may be NULLPTR, depending on the layer type.
+     *
+     * \since QGIS 3.18
+     */
+    virtual QgsMapLayerElevationProperties *elevationProperties() { return nullptr; }
+
   public slots:
 
     /**
@@ -1245,6 +1280,14 @@ class CORE_EXPORT QgsMapLayer : public QObject
      */
     void triggerRepaint( bool deferredUpdate = false );
 
+    /**
+     * Will advise any 3D maps that this layer requires to be updated in the scene.
+     * Will emit a request3DUpdate() signal.
+     *
+     * \since QGIS 3.18
+     */
+    void trigger3DUpdate();
+
     /**
      * Triggers an emission of the styleChanged() signal.
      * \since QGIS 2.16
@@ -1292,6 +1335,14 @@ class CORE_EXPORT QgsMapLayer : public QObject
     % End
 #endif
 
+    /**
+     * Returns the parent project if this map layer is added to a project.
+     * Otherwise returns NULLPTR
+     *
+     * \since QGIS 3.18
+     */
+    QgsProject *project() const;
+
   signals:
 
     /**
@@ -1332,6 +1383,16 @@ class CORE_EXPORT QgsMapLayer : public QObject
     //! Signal emitted when the blend mode is changed, through QgsMapLayer::setBlendMode()
     void blendModeChanged( QPainter::CompositionMode blendMode );
 
+    /**
+     * Emitted when the layer's opacity is changed, where \a opacity is a value between 0 (transparent)
+     * and 1 (opaque).
+     * \see setOpacity()
+     * \see opacity()
+     * \note Prior to QGIS 3.18, this signal was available for vector layers only
+     * \since QGIS 3.18
+     */
+    void opacityChanged( double opacity );
+
     /**
      * Signal emitted when renderer is changed.
      * \see styleChanged()
@@ -1359,6 +1420,13 @@ class CORE_EXPORT QgsMapLayer : public QObject
      */
     void renderer3DChanged();
 
+    /**
+     * Signal emitted when a layer requires an update in any 3D maps.
+     *
+     * \since QGIS 3.18
+     */
+    void request3DUpdate();
+
     /**
      * Emitted whenever the configuration is changed. The project listens to this signal
      * to be marked as dirty.
@@ -1424,10 +1492,16 @@ class CORE_EXPORT QgsMapLayer : public QObject
      */
     void isValidChanged();
 
+    /**
+     * Emitted when a custom property of the layer has been changed or removed.
+     *
+     * \since QGIS 3.18
+     */
+    void customPropertyChanged( const QString &key );
 
   private slots:
 
-    void onNotifiedTriggerRepaint( const QString &message );
+    void onNotified( const QString &message );
 
   protected:
 
@@ -1595,6 +1669,13 @@ class CORE_EXPORT QgsMapLayer : public QObject
      */
     bool mShouldValidateCrs = true;
 
+    /**
+     * Layer opacity.
+     *
+     * \since QGIS 3.18
+     */
+    double mLayerOpacity = 1.0;
+
   private:
 
     virtual QString baseURI( PropertyType type ) const;
