@@ -21,12 +21,12 @@
 
 /**
  * \ingroup core
-  Base class for plugin layers. These can be implemented by plugins
-  and registered in QgsPluginLayerRegistry.
-
-  In order to be readable from project files, they should set these attributes in layer DOM node:
-   "type" = "plugin"
-   "name" = "your_layer_type"
+ * \brief Base class for plugin layers. These can be implemented by plugins
+ *  and registered in QgsPluginLayerRegistry.
+ *
+ *  In order to be readable from project files, they should set these attributes in layer DOM node:
+ *  "type" = "plugin"
+ *  "name" = "your_layer_type"
  */
 class CORE_EXPORT QgsPluginLayer : public QgsMapLayer
 {
