@@ -46,9 +46,10 @@
 #include "qgsprojectmetadata.h"
 #include "qgstranslationcontext.h"
 #include "qgsprojecttranslator.h"
-#include "qgsattributeeditorelement.h"
 #include "qgscolorscheme.h"
 #include "qgssettings.h"
+#include "qgspropertycollection.h"
+
 
 class QFileInfo;
 class QDomDocument;
@@ -77,11 +78,13 @@ class QgsProjectViewSettings;
 class QgsProjectDisplaySettings;
 class QgsProjectTimeSettings;
 class QgsAnnotationLayer;
+class QgsAttributeEditorContainer;
+class QgsPropertyCollection;
 
 
 /**
  * \ingroup core
- * Encapsulates a QGIS project, including sets of map layers and their styles,
+ * \brief Encapsulates a QGIS project, including sets of map layers and their styles,
  * layouts, annotations, canvases, etc.
  *
  * QgsProject is available both as a singleton (QgsProject::instance()) and for use as
@@ -545,7 +548,7 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      *
      * \param scope entry scope (group) name
      * \param key entry key name. Keys are '/'-delimited entries, implying a hierarchy of keys and corresponding values.
-     * \param default value to return if the specified \a key does not exist within the \a scope.
+     * \param def default value to return if the specified \a key does not exist within the \a scope.
      * \param ok set to TRUE if key exists and has been successfully retrieved as a double
      *
      * \returns entry value as double from \a scope given its \a key
@@ -557,7 +560,7 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
      *
      * \param scope entry scope (group) name
      * \param key entry key name. Keys are '/'-delimited entries, implying a hierarchy of keys and corresponding values.
-     * \param default value to return if the specified \a key does not exist within the \a scope.
+     * \param def default value to return if the specified \a key does not exist within the \a scope.
      * \param ok set to TRUE if key exists and has been successfully retrieved as a boolean
      *
      * \returns entry value as boolean from \a scope given its \a key
@@ -2090,7 +2093,7 @@ class CORE_EXPORT QgsProject : public QObject, public QgsExpressionContextGenera
 Q_DECLARE_OPERATORS_FOR_FLAGS( QgsProject::ReadFlags )
 
 /**
- * Temporarily blocks QgsProject "dirtying" for the lifetime of the object.
+ * \brief Temporarily blocks QgsProject "dirtying" for the lifetime of the object.
  *
  * QgsProjectDirtyBlocker supports "stacked" blocking, so two QgsProjectDirtyBlockers created
  * for the same project will both need to be destroyed before the project can be dirtied again.
