@@ -33,7 +33,7 @@ class QgsFeedback;
 /**
  * \ingroup analysis
  * \class QgsRasterCalculatorEntry
- * Represents an individual raster layer/band number entry within a raster calculation.
+ * \brief Represents an individual raster layer/band number entry within a raster calculation.
  * \since QGIS 2.18
 */
 class ANALYSIS_EXPORT QgsRasterCalculatorEntry
@@ -71,7 +71,7 @@ class ANALYSIS_EXPORT QgsRasterCalculatorEntry
 
 /**
  * \ingroup analysis
- * Performs raster layer calculations.
+ * \brief Performs raster layer calculations.
 */
 class ANALYSIS_EXPORT QgsRasterCalculator
 {
@@ -87,8 +87,10 @@ class ANALYSIS_EXPORT QgsRasterCalculator
       ParserError = 4, //!< Error parsing formula
       MemoryError = 5, //!< Error allocating memory for result
       BandError = 6, //!< Invalid band number for input
+      CalculationError = 7, //!< Error occurred while performing calculation
     };
 
+
     /**
      * QgsRasterCalculator constructor.
      * \param formulaString formula for raster calculation
@@ -98,9 +100,13 @@ class ANALYSIS_EXPORT QgsRasterCalculator
      * \param nOutputColumns number of columns in output raster
      * \param nOutputRows number of rows in output raster
      * \param rasterEntries list of referenced raster layers
+     * \param transformContext coordinate transformation context
+     * \since QGIS 3.8
      */
     QgsRasterCalculator( const QString &formulaString, const QString &outputFile, const QString &outputFormat,
-                         const QgsRectangle &outputExtent, int nOutputColumns, int nOutputRows, const QVector<QgsRasterCalculatorEntry> &rasterEntries );
+                         const QgsRectangle &outputExtent, int nOutputColumns, int nOutputRows,
+                         const QVector<QgsRasterCalculatorEntry> &rasterEntries,
+                         const QgsCoordinateTransformContext &transformContext );
 
     /**
      * QgsRasterCalculator constructor.
@@ -112,10 +118,45 @@ class ANALYSIS_EXPORT QgsRasterCalculator
      * \param nOutputColumns number of columns in output raster
      * \param nOutputRows number of rows in output raster
      * \param rasterEntries list of referenced raster layers
-     * \since QGIS 2.10
+     * \param transformContext coordinate transformation context
+     * \since QGIS 3.8
      */
     QgsRasterCalculator( const QString &formulaString, const QString &outputFile, const QString &outputFormat,
-                         const QgsRectangle &outputExtent, const QgsCoordinateReferenceSystem &outputCrs, int nOutputColumns, int nOutputRows, const QVector<QgsRasterCalculatorEntry> &rasterEntries );
+                         const QgsRectangle &outputExtent, const QgsCoordinateReferenceSystem &outputCrs,
+                         int nOutputColumns, int nOutputRows,
+                         const QVector<QgsRasterCalculatorEntry> &rasterEntries,
+                         const QgsCoordinateTransformContext &transformContext );
+
+
+    /**
+    * QgsRasterCalculator constructor.
+    * \param formulaString formula for raster calculation
+    * \param outputFile output file path
+    * \param outputFormat output file format
+    * \param outputExtent output extent. CRS for output is taken from first entry in rasterEntries.
+    * \param nOutputColumns number of columns in output raster
+    * \param nOutputRows number of rows in output raster
+    * \param rasterEntries list of referenced raster layers
+    * \deprecated since QGIS 3.8, use the version with transformContext instead
+    */
+    Q_DECL_DEPRECATED QgsRasterCalculator( const QString &formulaString, const QString &outputFile, const QString &outputFormat,
+                                           const QgsRectangle &outputExtent, int nOutputColumns, int nOutputRows, const QVector<QgsRasterCalculatorEntry> &rasterEntries ) SIP_DEPRECATED;
+
+    /**
+     * QgsRasterCalculator constructor.
+     * \param formulaString formula for raster calculation
+     * \param outputFile output file path
+     * \param outputFormat output file format
+     * \param outputExtent output extent, CRS is specified by outputCrs parameter
+     * \param outputCrs destination CRS for output raster
+     * \param nOutputColumns number of columns in output raster
+     * \param nOutputRows number of rows in output raster
+     * \param rasterEntries list of referenced raster layers
+     * \deprecated since QGIS 3.8, use the version with transformContext instead
+     * \since QGIS 2.10
+     */
+    Q_DECL_DEPRECATED QgsRasterCalculator( const QString &formulaString, const QString &outputFile, const QString &outputFormat,
+                                           const QgsRectangle &outputExtent, const QgsCoordinateReferenceSystem &outputCrs, int nOutputColumns, int nOutputRows, const QVector<QgsRasterCalculatorEntry> &rasterEntries ) SIP_DEPRECATED;
 
     /**
      * Starts the calculation and writes a new raster.
@@ -139,17 +180,20 @@ class ANALYSIS_EXPORT QgsRasterCalculator
 
     /**
      * Opens the output driver and tests if it supports the creation of a new dataset
-      \returns nullptr on error and the driver handle on success*/
+     * \returns nullptr on error and the driver handle on success
+    */
     GDALDriverH openOutputDriver();
 
     /**
      * Opens the output file and sets the same geotransform and CRS as the input data
-      \returns the output dataset or nullptr in case of error*/
+     * \returns the output dataset or nullptr in case of error
+    */
     gdal::dataset_unique_ptr openOutputFile( GDALDriverH outputDriver );
 
     /**
      * Sets gdal 6 parameters array from mOutputRectangle, mNumOutputColumns, mNumOutputRows
-      \param transform double[6] array that receives the GDAL parameters*/
+     * \param transform double[6] array that receives the GDAL parameters
+    */
     void outputGeoTransform( double *transform ) const;
 
     //! Execute calculations on GPU
@@ -172,6 +216,8 @@ class ANALYSIS_EXPORT QgsRasterCalculator
 
     /***/
     QVector<QgsRasterCalculatorEntry> mRasterEntries;
+
+    QgsCoordinateTransformContext mTransformContext;
 };
 
 #endif // QGSRASTERCALCULATOR_H
