@@ -41,7 +41,7 @@ class QgsRasterDataProvider;
 
 /**
  * \ingroup core
- * Base class for processing modules.
+ * \brief Base class for processing modules.
  */
 class CORE_EXPORT QgsRasterPipe
 {
@@ -56,7 +56,7 @@ class CORE_EXPORT QgsRasterPipe
       ResamplerRole = 4,
       ProjectorRole = 5,
       NullerRole = 6,
-      HueSaturationRole = 7
+      HueSaturationRole = 7,
     };
 
     /**
@@ -72,7 +72,8 @@ class CORE_EXPORT QgsRasterPipe
 
     /**
      * Try to insert interface at specified index and connect
-     * if connection would fail, the interface is not inserted and false is returned */
+     * if connection would fail, the interface is not inserted and FALSE is returned
+    */
     bool insert( int idx, QgsRasterInterface *interface SIP_TRANSFER );
 #ifdef SIP_RUN
     % MethodCode
@@ -82,14 +83,15 @@ class CORE_EXPORT QgsRasterPipe
       // if insertion failed transfer ownership back to python
       PyObject *o = sipGetPyObject( a1, sipType_QgsRasterInterface );
       if ( o )
-        sipTransferBreak( o );
+        sipTransferTo( o, NULL );
     }
     % End
 #endif
 
     /**
      * Try to replace interface at specified index and connect
-     * if connection would fail, the interface is not inserted and false is returned */
+     * if connection would fail, the interface is not inserted and FALSE is returned
+    */
     bool replace( int idx, QgsRasterInterface *interface SIP_TRANSFER );
 
     /**
@@ -113,7 +115,8 @@ class CORE_EXPORT QgsRasterPipe
 
     /**
      * Set interface at index on/off
-     *  Returns true on success */
+     *  Returns TRUE on success
+    */
     bool setOn( int idx, bool on );
 
     //! Test if interface at index may be switched on/off
@@ -128,6 +131,34 @@ class CORE_EXPORT QgsRasterPipe
     QgsRasterProjector *projector() const;
     QgsRasterNuller *nuller() const;
 
+    /**
+     * Stage at which resampling occurs.
+     * \since QGIS 3.16
+     */
+    enum class ResamplingStage
+    {
+      //! Resampling occurs in ResamplingFilter
+      ResampleFilter,
+      //! Resampling occurs in Provider
+      Provider
+    };
+
+    /**
+     * Select which stage of the pipe should apply resampling.
+     *
+     * Provider resampling is only supported if provider sets
+     * ProviderHintCanPerformProviderResampling in providerCapabilities().
+     *
+     * \since QGIS 3.16
+     */
+    void setResamplingStage( ResamplingStage stage );
+
+    /**
+     * Returns which stage of the pipe should apply resampling
+     * \since QGIS 3.16
+     */
+    ResamplingStage resamplingStage() const { return mResamplingStage; }
+
   private:
 #ifdef SIP_RUN
     QgsRasterPipe( const QgsRasterPipe &pipe );
@@ -155,9 +186,11 @@ class CORE_EXPORT QgsRasterPipe
 
     /**
      * \brief Try to connect interfaces in pipe and to the provider at beginning.
-        Returns true if connected or false if connection failed */
+        Returns true if connected or false if connection failed
+    */
     bool connect( QVector<QgsRasterInterface *> interfaces );
 
+    ResamplingStage mResamplingStage = ResamplingStage::ResampleFilter;
 };
 
 #endif
