@@ -28,32 +28,35 @@
  * \ingroup core
  * \class QgsSettings
  *
- * This class is a composition of two QSettings instances:
+ * \brief This class is a composition of two QSettings instances:
+ *
  * - the main QSettings instance is the standard User Settings and
  * - the second one (Global Settings) is meant to provide read-only
  *   pre-configuration and defaults to the first one.
  *
  * For a given settings key, the function call to value(key, default) will return
  * the first existing setting in the order specified below:
- *  - User Settings
- *  - Global Settings
- *  - Default Value
+ *
+ * - User Settings
+ * - Global Settings
+ * - Default Value
  *
  * The path to the Global Settings storage can be set before constructing the QgsSettings
  * objects, with a static call to:
  * static bool setGlobalSettingsPath( QString path );
  *
  * QgsSettings provides some shortcuts to get/set namespaced settings from/to a specific section:
- *  - Core
- *  - Gui
- *  - Server
- *  - Plugins
- *  - Auth
- *  - App
- *  - Providers
- *  - Misc
  *
- * \since QGIS 3
+ * - Core
+ * - Gui
+ * - Server
+ * - Plugins
+ * - Auth
+ * - App
+ * - Providers
+ * - Misc
+ *
+ * \since QGIS 3.0
  */
 class CORE_EXPORT QgsSettings : public QObject
 {
@@ -71,19 +74,21 @@ class CORE_EXPORT QgsSettings : public QObject
       Auth,
       App,
       Providers,
+      Expressions,
       Misc
     };
 
     /**
-     * Construct a QgsSettings object for accessing settings of the application
+     * Constructs a QgsSettings object for accessing settings of the application
      * called application from the organization called organization, and with parent parent.
      */
     explicit QgsSettings( const QString &organization,
                           const QString &application = QString(), QObject *parent = nullptr );
 
     /**
-     * Construct a QgsSettings object for accessing settings of the application called application
+     * Constructs a QgsSettings object for accessing settings of the application called application
      * from the organization called organization, and with parent parent.
+     *
      * If scope is QSettings::UserScope, the QSettings object searches user-specific settings first,
      * before it searches system-wide settings as a fallback. If scope is QSettings::SystemScope,
      * the QSettings object ignores user-specific settings and provides access to system-wide settings.
@@ -98,11 +103,13 @@ class CORE_EXPORT QgsSettings : public QObject
                  const QString &application = QString(), QObject *parent = nullptr );
 
     /**
-     * Construct a QgsSettings object for accessing settings of the application called application
+     * Constructs a QgsSettings object for accessing settings of the application called application
      * from the organization called organization, and with parent parent.
+     *
      * If scope is QSettings::UserScope, the QSettings object searches user-specific settings first,
      * before it searches system-wide settings as a fallback. If scope is QSettings::SystemScope,
      * the QSettings object ignores user-specific settings and provides access to system-wide settings.
+     *
      * If format is QSettings::NativeFormat, the native API is used for storing settings. If format
      * is QSettings::IniFormat, the INI format is used.
      *
@@ -113,7 +120,7 @@ class CORE_EXPORT QgsSettings : public QObject
                  const QString &application = QString(), QObject *parent = nullptr );
 
     /**
-     * Construct a QgsSettings object for accessing the settings stored in the file called fileName,
+     * Constructs a QgsSettings object for accessing the settings stored in the file called fileName,
      * with parent parent. If the file doesn't already exist, it is created.
      *
      * If format is QSettings::NativeFormat, the meaning of fileName depends on the platform. On Unix,
@@ -122,14 +129,15 @@ class CORE_EXPORT QgsSettings : public QObject
      *
      * If format is QSettings::IniFormat, fileName is the name of an INI file.
      *
-     * Warning: This function is provided for convenience. It works well for accessing INI or .plist
+     * \warning This function is provided for convenience. It works well for accessing INI or .plist
      * files generated by Qt, but might fail on some syntaxes found in such files originated by
      * other programs. In particular, be aware of the following limitations:
-     *  - QgsSettings provides no way of reading INI "path" entries, i.e., entries with unescaped slash characters.
-     *    (This is because these entries are ambiguous and cannot be resolved automatically.)
-     *  - In INI files, QSettings uses the @ character as a metacharacter in some contexts, to encode
-     *     Qt-specific data types (e.g., \@Rect), and might therefore misinterpret it when it occurs
-     *     in pure INI files.
+     *
+     * - QgsSettings provides no way of reading INI "path" entries, i.e., entries with unescaped slash characters.
+     *   (This is because these entries are ambiguous and cannot be resolved automatically.)
+     * - In INI files, QSettings uses the @ character as a metacharacter in some contexts, to encode
+     *   Qt-specific data types (e.g., \@Rect), and might therefore misinterpret it when it occurs
+     *   in pure INI files.
      */
     QgsSettings( const QString &fileName, QSettings::Format format, QObject *parent = nullptr );
 
@@ -172,7 +180,7 @@ class CORE_EXPORT QgsSettings : public QObject
     //! Returns a list of all key top-level groups (same as childGroups) but only for groups defined in global settings.
     QStringList globalChildGroups() const;
     //! Returns the path to the Global Settings QSettings storage file
-    static QString globalSettingsPath() { return sGlobalSettingsPath; }
+    static QString globalSettingsPath();
     //! Sets the Global Settings QSettings storage file
     static bool setGlobalSettingsPath( const QString &path );
     //! Adds prefix to the current group and starts reading from an array. Returns the size of the array.
@@ -257,7 +265,7 @@ class CORE_EXPORT QgsSettings : public QObject
       if ( metaEnum.isValid() )
       {
         // read as string
-        QByteArray ba = value( key, metaEnum.valueToKey( defaultValue ), section ).toString().toUtf8();
+        QByteArray ba = value( key, metaEnum.valueToKey( static_cast<int>( defaultValue ) ), section ).toString().toUtf8();
         const char *vs = ba.data();
         v = static_cast<T>( metaEnum.keyToValue( vs, &ok ) );
         if ( ok )
@@ -300,7 +308,7 @@ class CORE_EXPORT QgsSettings : public QObject
       Q_ASSERT( metaEnum.isValid() );
       if ( metaEnum.isValid() )
       {
-        setValue( key, metaEnum.valueToKey( value ), section );
+        setValue( key, metaEnum.valueToKey( static_cast<int>( value ) ), section );
       }
       else
       {
@@ -329,7 +337,7 @@ class CORE_EXPORT QgsSettings : public QObject
         QgsDebugMsg( QStringLiteral( "Invalid metaenum. Enum probably misses Q_ENUM or Q_FLAG declaration." ) );
       }
 
-      T v;
+      T v = defaultValue;
       bool ok = false;
 
       if ( metaEnum.isValid() )
@@ -410,8 +418,6 @@ class CORE_EXPORT QgsSettings : public QObject
     void clear();
 
   private:
-
-    static QString sGlobalSettingsPath;
     void init();
     QString sanitizeKey( const QString &key ) const;
     QSettings *mUserSettings = nullptr;
