@@ -67,7 +67,10 @@ class ShellScintilla(QgsCodeEditorPython, code.InteractiveInterpreter):
         self.displayPrompt(self.continuationLine)
 
         for line in _init_commands:
-            self.runsource(line)
+            try:
+                self.runsource(line)
+            except ModuleNotFoundError:
+                pass
 
         self.history = []
         self.softHistory = ['']
@@ -530,7 +533,8 @@ class ShellScintilla(QgsCodeEditorPython, code.InteractiveInterpreter):
         self.displayPrompt(self.continuationLine)
 
     def write(self, txt):
-        sys.stderr.write(txt)
+        if sys.stderr:
+            sys.stderr.write(txt)
 
     def writeCMD(self, txt):
         if sys.stdout:
@@ -573,6 +577,13 @@ class HistoryDialog(QDialog, Ui_HistoryDialogPythonConsole):
         self.listView.doubleClicked.connect(self._runHistory)
         self.reloadHistory.clicked.connect(self._reloadHistory)
         self.saveHistory.clicked.connect(self._saveHistory)
+        self.runHistoryButton.clicked.connect(self._executeSelectedHistory)
+
+    def _executeSelectedHistory(self):
+        items = self.listView.selectionModel().selectedIndexes()
+        items.sort()
+        for item in items:
+            self.parent.runCommand(item.data(Qt.DisplayRole))
 
     def _runHistory(self, item):
         cmd = item.data(Qt.DisplayRole)
@@ -583,6 +594,7 @@ class HistoryDialog(QDialog, Ui_HistoryDialogPythonConsole):
 
     def _reloadHistory(self):
         self.model.clear()
+        item = None
         for i in self.parent.history:
             item = QStandardItem(i)
             if sys.platform.startswith('win'):
@@ -591,6 +603,8 @@ class HistoryDialog(QDialog, Ui_HistoryDialogPythonConsole):
 
         self.listView.setModel(self.model)
         self.listView.scrollToBottom()
+        if item:
+            self.listView.setCurrentIndex(self.model.indexFromItem(item))
 
     def _deleteItem(self):
         itemsSelected = self.listView.selectionModel().selectedIndexes()
