@@ -21,10 +21,6 @@ __author__ = 'Victor Olaya'
 __date__ = 'November 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '2f64a3c4e74022c5555ee19862f034ffacc2b5fe'
-
 from qgis.core import (QgsProcessing,
                        QgsProcessingException,
                        QgsProcessingParameterFeatureSource,
@@ -46,7 +42,7 @@ class OgrToPostGis(GdalAlgorithm):
     SHAPE_ENCODING = 'SHAPE_ENCODING'
     GTYPE = 'GTYPE'
     GEOMTYPE = ['', 'NONE', 'GEOMETRY', 'POINT', 'LINESTRING', 'POLYGON', 'GEOMETRYCOLLECTION', 'MULTIPOINT',
-                'MULTIPOLYGON', 'MULTILINESTRING']
+                'MULTIPOLYGON', 'MULTILINESTRING', 'CIRCULARSTRING', 'COMPOUNDCURVE', 'CURVEPOLYGON', 'MULTICURVE', 'MULTISURFACE']
     S_SRS = 'S_SRS'
     T_SRS = 'T_SRS'
     A_SRS = 'A_SRS'
@@ -61,7 +57,7 @@ class OgrToPostGis(GdalAlgorithm):
     PRIMARY_KEY = 'PRIMARY_KEY'
     GEOCOLUMN = 'GEOCOLUMN'
     DIM = 'DIM'
-    DIMLIST = ['2', '3']
+    DIMLIST = ['2', '3', '4']
     SIMPLIFY = 'SIMPLIFY'
     SEGMENTIZE = 'SEGMENTIZE'
     SPAT = 'SPAT'
@@ -240,22 +236,22 @@ class OgrToPostGis(GdalAlgorithm):
         simplify = self.parameterAsString(parameters, self.SIMPLIFY, context)
         segmentize = self.parameterAsString(parameters, self.SEGMENTIZE, context)
         spat = self.parameterAsExtent(parameters, self.SPAT, context)
-        clip = self.parameterAsBool(parameters, self.CLIP, context)
+        clip = self.parameterAsBoolean(parameters, self.CLIP, context)
         include_fields = self.parameterAsFields(parameters, self.FIELDS, context)
         fields_string = '-select "' + ','.join(include_fields) + '"'
         where = self.parameterAsString(parameters, self.WHERE, context)
         wherestring = '-where "' + where + '"'
         gt = self.parameterAsString(parameters, self.GT, context)
-        overwrite = self.parameterAsBool(parameters, self.OVERWRITE, context)
-        append = self.parameterAsBool(parameters, self.APPEND, context)
-        addfields = self.parameterAsBool(parameters, self.ADDFIELDS, context)
-        launder = self.parameterAsBool(parameters, self.LAUNDER, context)
+        overwrite = self.parameterAsBoolean(parameters, self.OVERWRITE, context)
+        append = self.parameterAsBoolean(parameters, self.APPEND, context)
+        addfields = self.parameterAsBoolean(parameters, self.ADDFIELDS, context)
+        launder = self.parameterAsBoolean(parameters, self.LAUNDER, context)
         launderstring = "-lco LAUNDER=NO"
-        index = self.parameterAsBool(parameters, self.INDEX, context)
+        index = self.parameterAsBoolean(parameters, self.INDEX, context)
         indexstring = "-lco SPATIAL_INDEX=OFF"
-        skipfailures = self.parameterAsBool(parameters, self.SKIPFAILURES, context)
-        promotetomulti = self.parameterAsBool(parameters, self.PROMOTETOMULTI, context)
-        precision = self.parameterAsBool(parameters, self.PRECISION, context)
+        skipfailures = self.parameterAsBoolean(parameters, self.SKIPFAILURES, context)
+        promotetomulti = self.parameterAsBoolean(parameters, self.PROMOTETOMULTI, context)
+        precision = self.parameterAsBoolean(parameters, self.PRECISION, context)
         options = self.parameterAsString(parameters, self.OPTIONS, context)
 
         arguments = []
@@ -264,7 +260,7 @@ class OgrToPostGis(GdalAlgorithm):
         if len(shapeEncoding) > 0:
             arguments.append('--config')
             arguments.append('SHAPE_ENCODING')
-            arguments.append('"' + shapeEncoding + '"')
+            arguments.append(shapeEncoding)
         arguments.append('-f')
         arguments.append('PostgreSQL')
         arguments.append('PG:' + self.getConnectionString(parameters, context))
