@@ -21,11 +21,8 @@ __author__ = 'Alexander Bruy'
 __date__ = 'September 2013'
 __copyright__ = '(C) 2013, Alexander Bruy'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '8a899c8758b30adc3f8d6df682a5702dd3bfe6f0'
-
-from qgis.core import (QgsProcessing,
+from qgis.core import (QgsProcessingAlgorithm,
+                       QgsProcessingException,
                        QgsProcessingParameterRasterLayer,
                        QgsProcessingParameterBand,
                        QgsProcessingParameterBoolean,
@@ -37,7 +34,6 @@ from processing.tools.system import isWindows
 
 
 class gdal2xyz(GdalAlgorithm):
-
     INPUT = 'INPUT'
     BAND = 'BAND'
     CSV = 'CSV'
@@ -51,6 +47,7 @@ class gdal2xyz(GdalAlgorithm):
                                                             self.tr('Input layer')))
         self.addParameter(QgsProcessingParameterBand(self.BAND,
                                                      self.tr('Band number'),
+                                                     1,
                                                      parentLayerParameterName=self.INPUT))
         self.addParameter(QgsProcessingParameterBoolean(self.CSV,
                                                         self.tr('Output comma-separated values'),
@@ -74,22 +71,22 @@ class gdal2xyz(GdalAlgorithm):
     def commandName(self):
         return 'gdal2xyz'
 
+    def flags(self):
+        return super().flags() | QgsProcessingAlgorithm.FlagDisplayNameIsLiteral
+
     def getConsoleCommands(self, parameters, context, feedback, executing=True):
         arguments = []
         arguments.append('-band')
         arguments.append(str(self.parameterAsInt(parameters, self.BAND, context)))
 
-        if self.parameterAsBool(parameters, self.CSV, context):
+        if self.parameterAsBoolean(parameters, self.CSV, context):
             arguments.append('-csv')
 
-        arguments.append(self.parameterAsRasterLayer(parameters, self.INPUT, context).source())
+        raster = self.parameterAsRasterLayer(parameters, self.INPUT, context)
+        if raster is None:
+            raise QgsProcessingException(self.invalidRasterError(parameters, self.INPUT))
+
+        arguments.append(raster.source())
         arguments.append(self.parameterAsFileOutput(parameters, self.OUTPUT, context))
 
-        commands = []
-        if isWindows():
-            commands = ['cmd.exe', '/C ', 'gdal2xyz.bat',
-                        GdalUtils.escapeAndJoin(arguments)]
-        else:
-            commands = ['gdal2xyz.py', GdalUtils.escapeAndJoin(arguments)]
-
-        return commands
+        return [self.commandName() + ('.bat' if isWindows() else '.py'), GdalUtils.escapeAndJoin(arguments)]
